#!/usr/bin/perl -Tw
#
# $Id: wwwboard.pl,v 1.11 2002/01/15 20:34:03 gellyfish Exp $
#
# $Log: wwwboard.pl,v $
# Revision 1.11  2002/01/15 20:34:03  gellyfish
# * Removed nasty global variables prior to refactoring
#
# Revision 1.10  2001/12/01 19:45:22  gellyfish
# * Tested everything with 5.004.04
# * Replaced the CGI::Carp with local variant
#
# Revision 1.9  2001/11/26 13:40:05  nickjc
# Added \Q \E around variables in regexps where metacharacters in the
# variables shouldn't be interpreted by the regex engine.
#
# Revision 1.8  2001/11/25 11:39:40  gellyfish
# * add missing use vars qw($DEBUGGING) from most of the files
# * sundry other compilation failures
#
# Revision 1.7  2001/11/25 09:45:25  gellyfish
# * Added stub FAQ (no answers just questions)
# * More toshing on wwwboard.pl
#
# Revision 1.6  2001/11/24 20:01:22  gellyfish
# Sundry refactoring
#
# Revision 1.5  2001/11/24 11:59:58  gellyfish
# * documented strfime date formats is various places
# * added more %ENV cleanup
# * spread more XHTML goodness and CSS stylesheet
# * generalization in wwwadmin.pl
# * sundry tinkering
#
# Revision 1.4  2001/11/19 09:21:44  gellyfish
# * added allow_html functionality
# * fixed potential for pre lock clobbering in guestbook
# * some XHTML toshing
#
# Revision 1.3  2001/11/13 20:35:14  gellyfish
# Added the CGI::Carp workaround
#
# Revision 1.2  2001/11/11 17:55:27  davorg
# Small amount of post-import tidying :)
#
#

use strict;
use CGI qw(:standard);
use Fcntl qw(:DEFAULT :flock);

# It appears that the 'SEEK_*' things arent defined everywhere

BEGIN
{
   sub SEEK_SET() { 0;}
};

use POSIX qw(strftime);

use vars qw($DEBUGGING);

my $VERSION = '1.0';

# Configuration

#
# $DEBUGGING must be set in a BEGIN block in order to have it be set before
# the program is fully compiled.
# This should almost certainly be set to 0 when the program is 'live'
#

BEGIN
{
   $DEBUGGING = 1;
}
   
my $basedir = '/var/www/nms-test/wwwboard';
my $baseurl = 'http://nms-test/wwwboard';
my $cgi_url = 'http://nms-test/cgi-bin/wwwboard.pl';

my $mesgdir  = 'messages';
my $datafile = 'data.txt';
my $mesgfile = 'wwwboard.html';
my $faqfile  = 'faq.html';

my $ext = 'html';

# $title is the title that will be displayed on the program generated pages

my $title = "NMS WWWBoard Version $VERSION";

# $style is the URL of a CSS stylesheet which will be used for script
# generated messages.  This probably want's to be the same as the one
# that you use for all the other pages.  This should be a local absolute
# URI fragment.

my $style = '/css/nms.css';

# $emulate_matts_code determines whether the program should behave exactly
# like the original wwwboard program.  It should be set to 1 if you
# want to emulate the original program - this is recommended if you are
# replacing an existing installation with this program.  If it is set to 0
# then potentially it will not work with files produced by the original
# version - this is recommended for people installing this for the first time.

my $emulate_matts_code = 1;

# If $show_faq is set to 1 then the link for the WWWBoard FAQ will be shown
# on the generated pages.

my $show_faq     = 1;

# If $allow_html is set to 1 then HTML will not be removed from the body of
# the message.  Be warned however that setting this is a possible security
# risk allowing malicious defacement of the page and possible cross page
# scripting attacks on this or other site.

my $allow_html   = 1;

# If $quote_text is set to 1 then the text of the original message will
# be placed in the "Post Followup" text box

my $quote_text   = 1;

# $quote_char is the string that will be prepended to each quoted line
# if $quote_text described above is set to 1

my $quote_char  = ':';

# If $quote_html is set to 1 then if $quote_text is set to 1 above the
# original HTML in the post will be retained in the followup - otherwise
# it will be removed

my $quote_html = 0; 

my $subject_line = 0;	# 0 = Quote Subject Editable
                        # 1 = Quote Subject UnEditable; 
                        # 2 = Don't Quote Subject, Editable.

# If $use_time is set to 1 then the dates will be displayed as
# "$time_fmt $date_fmt" - see note below about date formats

my $use_time        = 1;

# $date_fmt and $time_fmt describe the format of the dates that
# will output - the replacement parameters you can use here are:
#
# %A - the full name of the weekday according to the current locale
# %B - the full name of the month according to the current local
# %m - the month as a number
# %d - the day of the month as a number
# %D - the date in the form %m/%d/%y (i.e. the US format )
# %y - the year as a number without the century
# %Y - the year as a number including the century
# %H - the hour as number in the 24 hour clock
# %M - the minute as a number
# %S - the seconds as a number
# %T - the time in 24 hour format (%H:%M:%S)
# %Z - the time zone (full name or abbreviation)

my $date_fmt = '%d/%m/%y';
my $time_fmt = '%T';

# $show_poster_ip determines whether the client IP address of the poster
# is displayed in the message.

my $show_poster_ip  = 1;

my $enforce_max_len = 0;   # 2 = YES, error; 1 = YES, truncate; 0 = NO

# These are the maximum lengths of the data allowed in the various
# fields - they must be supplied if $enforce_max_len is not 0

my %max_len = ('name'        => 50,
	       'email'       => 70,
	       'subject'     => 80,
	       'url'         => 150,
	       'url_title'   => 80,
	       'img'         => 150,
	       'body'        => 3000,
	       'origsubject' => 80,
	       'origname'    => 50,
	       'origemail'   => 70,
	       'origdate'    => 50);

# $strict_image if set to 1 will require that an image url if supplied
# must end in one of the common suffixes as defined in @image_suffixes

my $strict_image = 1;

# @images_suffixes is a list of the extensions that the image URL must end
# with if $strict_image above is set to 1 - this is to minimize a potential
# for the entry of entry of a malicious URL that may be used to attack
# another host or disfigure this web page.

my @image_suffixes = qw(png jpe?g gif);

# End configuration

# We need finer control over what gets to the browser and the CGI::Carp
# set_message() is not available everywhere :(
# This is basically the same as what CGI::Carp does inside but simplified
# for our purposes here.

BEGIN
{
   sub fatalsToBrowser
   {
      my ( $message ) = @_;

      if ( $main::DEBUGGING )
      {
         $message =~ s/</&lt;/g;
         $message =~ s/>/&gt;/g;
      }
      else
      {
         $message = '';
      }
      
      my ( $pack, $file, $line, $sub ) = caller(1);
      my ($id ) = $file =~ m%([^/]+)$%;

      return undef if $file =~ /^\(eval/;

      print "Content-Type: text/html\n\n";

      print <<EOERR;
<html>
  <head>
    <title>Error</title>
  </head>
  <body>
     <h1>Application Error</h1>
     <p>
     An error has occurred in the program
     </p>
     <p>
     $message
     </p>
  </body>
</html>
EOERR
     die @_;
   };

   $SIG{__DIE__} = \&fatalsToBrowser;
}   

if ( $use_time ) {
   $date_fmt = "$time_fmt $date_fmt";
}


my $id = get_number();


my $Form = parse_form();

my $variables = get_variables($Form,$id);

new_file($variables);
main_page($variables);

thread_pages($variables);

return_html($variables);
increment_num($variables); # Is this necessary with the current implementation 
                           # of get_number() ?

sub get_number {
  sysopen(NUMBER, "$basedir/$datafile", O_RDWR|O_CREAT)
    || die "Can't open number file: $!\n";
  flock(NUMBER, LOCK_EX)
    || die "Can't lock number file: $!\n";
  my $num = <NUMBER> || 0;

  if ($num =~ /^(\d)+$/) {
    $num = $1;
  } else {
    $num = 0;
  }

  if ($num == 999999 || $num !~ /^\d+$/)  {
    $num = 1;
  } else {
    $num++;
  }

  seek(NUMBER, SEEK_SET, 0);
  truncate NUMBER, 0;
  print NUMBER $num;

  close NUMBER;

  return $num;
}

sub parse_form {
  my %Form;

  # Are we sure this is valid syntax in 5.004.04 ?

  foreach my $param ( keys %max_len , 'followup' )
  {
     $Form{$param} = param($param) || '';
  }

  if ($enforce_max_len) {
    foreach (keys %max_len) {
      if (length($Form{$_}) > $max_len{$_}) {
	if ($enforce_max_len == 2) {
	  &error('field_size');
	} else {
	  $Form{$_} = substr($Form{$_}, 0, $max_len{$_});
	}
      }
    }
  }
  return \%Form;
}

###############
# Get Variables

sub get_variables {

  my ( $Form,$id ) = @_;

  my $variables = {id   => $id,
                   Form => $Form}; # Just in case ;-}

  my @followup_num;

  if ($Form->{followup}) {
    $variables->{followup} = 1;
    @followup_num = split(/,/, $Form->{followup});

    my %fcheck;
    foreach my $fn (@followup_num) {
      error('followup_data') if $fn !~ /^\d+$/ || $fcheck{$fn};
      $fcheck{$fn}++;
    }

    # WTF!

    @followup_num = keys %fcheck;

    $variables->{followups} = \@followup_num;
    $variables->{num_followups} = scalar @followup_num;
    $variables->{last_message} = pop(@{$variables->{followups}});
    $variables->{origdate} = $Form->{origdate};
    $variables->{origname} = $Form->{origname};
    $variables->{origsubject} = $Form->{origsubject};
  } else {
    $variables->{followup} = $variables->{num_followups} = 0;
  }

  if (my $name = $Form->{name}) {
    $name =~ s/\"//g;
    $name =~ s/<//g;
    $name =~ s/>//g;
    $name =~ s/\&//g;

    $variables->{name} = $name;
  } else {
    error('no_name');
  }

  if ($Form->{email} =~ /(.*\@.*\..*)/) {
    $variables->{email} = $1;
  }

  if ($Form->{subject}) {
    $variables->{subject} = escape_html($Form->{subject});
  } else {
    error('no_subject');
  }

  if ($Form->{'url'} =~ /(.*\:.*\..*)/ && $Form->{'url_title'}) {
    $variables->{message_url} = $1;
    $variables->{message_url_title} = $Form->{'url_title'};
  }

  my $image_suffixes = '.+';

  if ( $strict_image )
  {
     $image_suffixes = join '|', @image_suffixes;

     $image_suffixes = "($image_suffixes)";
  }
  if ($Form->{'img'} =~ m%^(.+tp://.*\.$image_suffixes)$%) {
    $variables->{message_img} = $1;
  }

  if (my $body = $Form->{'body'}) {
    $body = strip_html($body,$allow_html);
    $body = "<p>$body</p>";
    $body =~ s/\cM//g;
    $body =~ s|\n\n|</p><p>|g;
    $body =~ s%\n%<br />%g;

    # I'm not entirely sure if this is what is actually meant :
    # it would allow someone to subvert $allow_html by putting escaped stuff
    # in the message and having the script expand it.

    $variables->{body} = unescape_html($body);
     
  } else {
    error('no_body');
  }

  if ($quote_text) 
  {
    my $hidden_body = $variables->{body};

    if ( $quote_html ) 
    {
       $hidden_body = escape_html($hidden_body);
    }
    else 
    {
       $hidden_body = strip_html($hidden_body,$allow_html);
    }

    $variables->{hidden_body} = $hidden_body;
   
   }

   $variables->{date} = strftime($date_fmt , localtime());

   return $variables;
}

#####################
# New File Subroutine

sub new_file {

  my ($variables) = @_;

  open(NEWFILE,">$basedir/$mesgdir/$variables->{id}.$ext")
    || die "$! [$basedir/$mesgdir/$variables->{id}.$ext]";

  my $faq = $show_faq ? qq( [ <a href="$baseurl/$faqfile">FAQ</a> ]) : '';
  my $print_name = $variables->{email} ? 
            qq(<a href="mailto:$variables->{email}">$variables->{name}</a> ) : 
            $variables->{name};
  my $ip = $show_poster_ip ? "($ENV{REMOTE_ADDR})" : '';
  my $pr_follow = $variables->{followup} ? 
    qq(<p>In Reply to:
       <a href="$variables->{last_message}.$ext">$variables->{origsubject}</a> posted by ) .
	 $variables->{origemail} ? 
         qq(<a href="$variables->{origemail}">$variables->{origname}</a>) 
	   : $variables->{origname} . "on $variables->{origdate}:</p>" : '';
  my $img = $variables->{message_img} ? 
    qq(<p align="center"><img src="$variables->{message_img}"></p>\n) : '';
  my $url = $variables->{message_url} ? 
    qq(<ul><li><a href="$variables->{message_url}">$variables->{message_url_title}</a></li></ul><br>) :
      '';

  print NEWFILE <<END_HTML;
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html>
  <head>
    <title>$variables->{subject}</title>
    <link rel="stylesheet" type="text/css" href="$style" />
  </head>
  <body>
    <h1 align="center">$variables->{subject}</h1>
    <hr />
    <p align="center">
      [ <a href="#followups">Follow Ups</a> ]
      [ <a href="#postfp">Post Followup</a> ]
      [ <a href="$baseurl/$mesgfile">$title</a> ]
      $faq
    </p>

  <hr />
  <p>Posted by $print_name $ip on $variables->{date}</p>

  $variables->{followup} $img

  $variables->{body}<br />$url

  <hr />
  <p><a name="followups">Follow Ups:</a><br />
  <ul><!--insert: $variables->{id}-->
  </ul><!--end: $variables->{id}-->
  <br /><hr />
  <p><a name="postfp">Post a Followup</a></p>
  <form method=POST action="$cgi_url">
END_HTML

  my $follow_ups = @{$variables->{followups}}  ? 
                    join ',', @{$variables->{followup_num}} : '';

  $follow_ups .= ',' if length($follow_ups);

  my $id = $variables->{id};

  print NEWFILE qq(<input type="hidden" name="followup" value="$follow_ups$id" />);

  print NEWFILE qq(<input type=hidden name="origname" value="$variables->{name}" />);
  print NEWFILE qq(<input type=hidden name="origemail" value="$variables->{email}" />)
    if $variables->{email};

  print NEWFILE <<END_HTML;
<input type="hidden" name="origsubject" value="$variables->{subject}" />
<input type="hidden" name="origdate" value="$variables->{date}" />
<table>
<tr>
<td>Name:</td>
<td><input type="text" name="name" size="50" /></td>
</tr>
<tr>
<td>E-Mail:</td>
<td><input type="text" name="email" size="50" /></td>
</tr>
END_HTML

  my $subject = $variables->{subject};

  $subject = 'Re: ' . $subject unless $subject =~ /^Re:/i;

  if ($subject_line == 1) {
    print NEWFILE qq(<input type="hidden" name="subject" value="$subject" />\n);
    print NEWFILE "<tr><td>Subject:</td><td><b>$subject</b></td></tr>\n";
  } elsif ($subject_line == 2) {
    print NEWFILE qq(<tr><td>Subject:</td><td><input type="text" name="subject" size="50"></td></tr>\n);
  } else {
    print NEWFILE qq(<tr><td>Subject:</td><td><input type="text" name="subject"value="$subject" size="50"></td></tr>\n);
  }
  print NEWFILE "<tr><td>Comments:</td>\n";
  print NEWFILE qq(<td><textarea name="body" COLS="50" ROWS="10">\n);
  if ($quote_text) {
    print NEWFILE map { "$quote_char $_\n" } split /\n/, $variables->{hidden_body};
    print NEWFILE "\n";
  }
  print NEWFILE "</textarea></td></tr>\n";
  print NEWFILE <<END_HTML;
<tr>
<td>Optional Link URL:</td>
<td><input type="text" name="url" size="50" /></td>
</tr>
<tr>
<td>Link Title:</td>
<td><input type="text" name="url_title" size="48" /></td>
</tr>
<tr>
<td>Optional Image URL:</td>
<td><input type="text" name="img" size="49" /></td>
</tr>
<tr>
<td colspan="2"><input type="submit" value="Submit Follow Up" /> 
<input type="reset" /></td>
</tr>
<hr />
<p align="center">
   [ <a href="#followups">Follow Ups</a> ] 
   [ <a href="#postfp">Post Followup</a> ] 
   [ <a href="$baseurl/$mesgfile">$title</a> ] 
   $faq
</p>
</body>
</html>
END_HTML
  close(NEWFILE);
}

###############################
# Main WWWBoard Page Subroutine

sub main_page {

   my ( $variables ) = @_;
  open(MAIN,"$basedir/$mesgfile") ||
    die "$! [$basedir/$mesgfile]";

  my @main = <MAIN>;
  close(MAIN);

  open(MAIN,">$basedir/$mesgfile") || die $!;

  my $id = $variables->{id};
  my $name = $variables->{name};
  my $subject = $variables->{subject};
  my $date = $variables->{date};

  if ($variables->{followup} == 0) {
    foreach (@main) {
      if (/<!--begin-->/) {
	print MAIN <<END_HTML;
<!--begin-->
<!--top: $id--><li><a href="$mesgdir/$id.$ext">$subject</a> - <b>$name</b> <i>$date</i>
(<!--responses: $id-->0)
<ul><!--insert: $id-->
</ul><!--end: $id-->
END_HTML
      } else {
	print MAIN $_;
      }
    }
  } else {
    foreach (@main) {
      my $work = 0;
      if (/<ul><!--insert: $variables->{last_message}-->/) {
	print MAIN <<END_HTML;
<ul><!--insert: $variables->{last_message}-->
<!--top: $id--><li><a href="$mesgdir/$id.$ext">$subject</a> - <b>$name</b> <i>$date</i>
(<!--responses: $id-->0)
<ul><!--insert: $id-->
</ul><!--end: $id-->
END_HTML
      } elsif (/\(<!--responses: (.*)-->(.*)\)/) {
	my $response_num = $1;
	my $num_responses = $2;
	$num_responses++;
	foreach my $followup_num (@{$variables->{followup}}) {
	  if ($followup_num == $response_num) {
	    print MAIN "(<!--responses: $followup_num-->$num_responses)\n";
	    $work = 1;
	  }
	}
	if ($work != 1) {
	  print MAIN $_;
	}
      } else {
	print MAIN $_;
      }
    }
  }
  close(MAIN);
}

############################################
# Add Followup Threading to Individual Pages
sub thread_pages {

  my ($variables) = @_;

  return unless $variables->{num_followups};

  my $id = $variables->{id};
  my $subject = $variables->{subject};
  my $name    = $variables->{name};
  my $date    = $variables->{date};

  foreach my $followup_num (@{$variables->{followup_num}}) {
    open(FOLLOWUP, "$basedir/$mesgdir/$followup_num.$ext")
      || die "$!";

    my @followup_lines = <FOLLOWUP>;
    close(FOLLOWUP);

    open(FOLLOWUP, "<+$basedir/$mesgdir/$followup_num.$ext")
      || die "$!";

    flock FOLLOWUP, LOCK_EX or die "Can't lock $!\n";

    seek FOLLOWUP, SEEK_SET,0;

    foreach (@followup_lines) {
      my $work = 0;
      if (/<ul><!--insert: $variables->{last_message}-->/) {
	print FOLLOWUP<<END_HTML;
<ul><!--insert: $variables->{last_message}-->
<!--top: $id--><li><a href="$id\.$ext">$subject</a> <b>$name</b> <i>$date</i>
(<!--responses: $id-->0)
<ul><!--insert: $id-->
</ul><!--end: $id-->
END_HTML
      } elsif (/\(<!--responses: (.*)-->(.*)\)/) {
	my $response_num = $1;
	my $num_responses = $2;
	$num_responses++;
	foreach $followup_num (@{$variables->{followups}}) {
	  if ($followup_num == $response_num) {
	    print FOLLOWUP "(<!--responses: $followup_num-->$num_responses)\n";
	    $work = 1;
	  }
	}
	if ($work != 1) {
	  print FOLLOWUP $_;
	}
      } else {
	print FOLLOWUP $_;
      }
    }
    close(FOLLOWUP);
  }
}

sub return_html {

  my ( $variables ) = @_;

  print header;

  my $url = $variables->{message_url} ? 
    qq(<p><b>Link:</b> <a href="$variables->{message_url}">$variables->{message_url_title}</a></p>) : '';
  my $img = $variables->{message_img} ? 
    qq(<p><b>Image:</b> <img src="$variables->{message_img}"></p>) : '';

  print <<END_HTML;
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html>
  <head>
    <title>Message Added: $variables->{subject}</title>
    <link rel="stylesheet" type="text/css" href="$style" />
  </head>
  <body>
    <h1 align="center">Message Added: $variables->{subject}</h1>
    <p>The following information was added to the message board:</p>
    <hr>
    <p><b>Name:</b> $variables->{name}<br>
      <b>E-Mail:</b> $variables->{email}<br>
      <b>Subject:</b> $variables->{subject}<br>
      <b>Body of Message:</b></p>
      <p>$variables->{body}</p>
    $url
    $img

    <p><b>Added on Date:</b> $variables->{date}</p>
    <hr>
    <p align="center">[ <a href="$baseurl/$mesgdir/$id.$ext">Go to Your Message</a> ] [ <a href="$baseurl/$mesgfile">$title</a> ]</p>
  </body>
</html>
END_HTML
}

sub increment_num {
  my ($variables) = @_;
  open(NUM,">$basedir/$datafile") || die $!;
  print NUM $id;
  close(NUM);
}

sub error {
  my $error = $_[0];

  print header;

  if ($error eq 'no_name') {
    print <<END_HTML;
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html>
  <head>
    <title>$title ERROR: No Name</title>
    <link rel="stylesheet" type="text/css" href="$style" />
  </head>
  <body><h1 align="center">ERROR: No Name</h1>
  <p>You forgot to fill in the 'Name' field in your posting.  Correct it 
    below and re-submit.  The necessary fields are: Name, Subject and 
    Message.</p>
  <hr>
END_HTML
    rest_of_form();
  } elsif ($error eq 'no_subject') {
    print <<END_HTML;
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html>
  <head>
    <title>$title ERROR: No Subject</title>
    <link rel="stylesheet" type="text/css" href="$style" />
  </head>
  <body><h1 align="center">ERROR: No Subject</h1>
  <p>You forgot to fill in the 'Subject' field in your posting.  Correct it 
  below and re-submit.  The necessary fields are: Name, Subject and 
  Message.</p>
  <hr>
END_HTML
    rest_of_form();
  } elsif ($error eq 'no_body') {
    print <<END_HTML;
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html>
<head>
<title>$title ERROR: No Message</title>
<link rel="stylesheet" type="text/css" href="$style" />
</head>
<body><align="center"><h1>ERROR: No Message</h1>
<p>You forgot to fill in the 'Message' field in your posting.  Correct it
below and re-submit.  The necessary fields are: Name, Subject and 
Message.</p>
<hr>
END_HTML
rest_of_form();
} elsif ($error eq 'field_size') {
printf <<END_HTML;
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html>
  <head>
    <title>$title ERROR: Field too Long</title>
    <link rel="stylesheet" type="text/css" href="$style" />
  </head>
  <body><h1 align="center">ERROR: Field too Long</h1>
  <p>One of the form fields in the message submission was too long.  The 
  following are the limits on the size of each field (in characters):</p>
  <ul>
    <li>Name: $max_len{'name'}</li>
    <li>E-Mail: $max_len{'email'}</li>
    <li>Subject: $max_len{'subject'}</li>
    <li>Body: $max_len{'body'}</li>
    <li>URL: $max_len{'url'}</li>
    <li>URL Title: $max_len{'url_title'}</li>
    <li>Image URL: $max_len{'img'}</li>
  </ul>
  <p>Please modify the form data and resubmit.</p>
  <hr>
END_HTML
     rest_of_form();
   } else {
     print "<p>ERROR!  Undefined.</p>";
   }
  exit;
}

sub rest_of_form {

  my ( $variables ) = @_;

  print qq(<form method="POST" action="$cgi_url">\n);

  my %Form = $variables->{Form};

  if ($variables->{followup} == 1) {
    print qq(<input type="hidden" name="origsubject" value="$Form{origsubject}" />\n);
    print qq(<input type="hidden" name="origname" value="$Form{origname}" />\n);
    print qq(<input type="hidden" name="origemail" value="$Form{origemail}" />\n);
    print qq(<input type="hidden" name="origdate" value="$Form{origdate}" />\n);
    print qq(<input type="hidden" name="followup" value="$Form{followup}" />\n);
  }
  print qq(Name: <input type="text" name="name" value="$Form{name}" size="50" /><br />\n);
  print qq(E-Mail: <input type="text" name="email" value="$Form{email}" size="50" /><p />\n);
  if ($subject_line == 1) {
    print qq(<input type="hidden" name="subject" value="$Form{subject}" />\n);
    print qq(Subject: <b>$Form{subject}</b><p />\n);
  } else {
    print qq(Subject: <input type="text" name="subject" value="$Form{subject}" size="50" /><p />\n);
   }
  
  $Form{'body'} = escape_html($Form{'body'});

  print "Message:<br>\n";
  print qq(<textarea COLS="50" ROWS="10" name="body">\n);

  print "$Form{'body'}\n";
  print "</textarea><p>\n";
  print qq(Optional Link URL: <input type=text name="url" value="$Form{'url'}" size="45" /><br />\n);
  print qq(Link Title: <input type="text" name="url_title" value="$Form{'url_title'}" size="50" /><br />\n);
  print qq(Optional Image URL: <input type="text" name="img" value="$Form{'img'}" size="45" /><p />\n);
  print qq(<input type="submit" value="Post Message" /> <input type="reset" />\n);
  print "</form>\n";
  print qq(<br /><hr size="7" width="75%" />\n);
  if ($show_faq) {
    print qq(<center>[ <a href="#followups">Follow Ups</a> ] [ <a href="#postfp">Post Followup</a> ] [ <a href="$baseurl/$mesgfile">$title</a> ] [ <a href="$baseurl/$faqfile">FAQ</a> ]</center>\n);
  } else {
    print qq(<center>[ <a href="#followups">Follow Ups</a> ] [ <a href="#postfp">Post Followup</a> ] [ <a href="$baseurl/$mesgfile">$title</a> ]</center>\n);
  }
  print "</body></html>\n";
}

# subroutine to crudely strip html from a text string
# ideally we would want to use HTML::Parser or somesuch.
# we will also implement any selective tag replacement here
# thus all user supplied input that will be displayed should
# be passed through this before being displayed.

sub strip_html
{
   my ( $comments,$allow_html ) = @_;

   $allow_html = defined $allow_html ? $allow_html : 0;

   $comments =~ s/(?:<[^>'"]*|".*?"|'.*?')+>//gs unless $allow_html;

   # remove any comments that could harbour an attempt at an SSI exploit
   # suggested by Pete Sargeant

   $comments =~ s/<!--.*?-->/ /gs;

   # mop up any stray start or end of comment tags.

   $comments = "<!-- -->$comments<!-- -->";

   return $comments;
}

# subroutine to escape the necessary characters to the appropriate HTML
# entities

use vars qw(%escape_html_map %unescape_html_map);

BEGIN
{
   %escape_html_map = ( '&' => '&amp;',
                        '<' => '&lt;',
                        '>' => '&gt;',
                        '"' => '&quot;',
                        "'" => '&#39;',
                      );

   while ( my ( $key, $value ) = each %escape_html_map )
   {
      $unescape_html_map{$value} = $key;
   }
}

sub escape_html {
  my $str = shift;
  my $chars = join '', keys %escape_html_map;
  $str =~ s/([\Q$chars\E])/$escape_html_map{$1}/g;
  return $str;
}

sub unescape_html {
  my $str = shift;
  my $pattern = join '|', map { quotemeta($_) } keys(%unescape_html_map);
  $str =~ s/($pattern)/$unescape_html_map{$1}/g;
  return $str;
}
